Learn R Programming

fluffy (version 1.0.0)

show_builtins: Show Builtin Rules

Description

Helper to show the builtin rules in a Registry, and their behaviour.

Usage

show_builtins(rules = c("all", "validation", "cross"))

Value

List of data.fames if rules = "all", otherwise a single data.frame, with information on the builtin rules for the rule type specified. The data.frame(s) have an attached class fluffy_rule_info.

Arguments

rules

Which builtin rules to show. "validation" for the standard schema/data validation rules, "cross" for the cross rules that check for consistency between rules, or "all" to show both.

See Also

Registry and add_rule.

Examples

Run this code
show_builtins()

Run the code above in your browser using DataLab